<?php
	/*
	*	E A S Y  RSS
	*	  Copyright IntervalNetwork
	*	  Version 1.0b - Decembre 2006
	*     Pour PHP v4
	*
	*	Ce script permet de lire les news au format xml+rss v1.0
	*	Il met en cache le resultat pour eviter de multiplier les requetes inutiles
	*	Il peut utiliser le cache si la connexion ne reponds pas temporairement
	*
	*	Utilisation :
	*	<?
	*		include "easy.rss.php";
	*		$flux = new EasyRSS;
	*		$feed = lire_xml("http://monsite.com/feed.xml", "2 weeks", "ASSOC", "item", array("title", "pubDate"));
	*		if (!$feed) echo "Erreur Parsing XML File";
	*		else print_r($feed);
	*	?>
	*
	*	Explication de la fonction lire_xml() :
	*		lire_xml(str url, str refresh, str result_type, str item, array champs)
	*			url : adresse du flux xml
	*			refresh : delai avant de ractualiser le cache (format : "valeur unit" ; ex: "1 month", "3 days", "2 weeks")
	*			result_type : cles du tableau de resultat : ASSOC (nom du champ) ou NUM (numro)
	*			item : nom des balises contenant les items
	*			champs : tableau contenant tous les champs  retourner dans le rsultat
	*/

	class EasyRSS {

		##
		#### C O N F I G U R A T I O N
		##
		var $temp_dir = "cache/rss"; // dossier temporaire
		var $trigger_errors = false; // afficher ou non les erreurs

		#### Fin de la configuration
		var $ready = false;

		function EasyRSS($dir="") {
			if (empty($dir)) $dir = $this->temp_dir;
			# Test de url_fopen
			if (!ini_get("allow_url_fopen")) {
				if ($this->trigger_errors) trigger_error("EasyRSS::__construct Erreur : vous n'avez pas l'autorisation de lire des fichiers distants");
			}
			# Test du repertoire temporaire
			else if (@!file_exists($dir) || @!is_dir($dir) || @!is_readable($dir)) {
				if ($this->trigger_errors) trigger_error("EasyRSS::__construct Erreur : le repertoire temporaire est invalide");
			}
			# Sortie
			else $this->ready = true;
			$this->temp_dir = $dir;
			return $this->ready;
		}

		function lire_xml($addr, $update=0, $result_type="ASSOC", $item="item", $champs=array("title", "link", "pubDate")) {

			if (!$this->ready) {
				if ($this->trigger_errors) trigger_error("EasyRSS::lire_xml Erreur : impossible d'utiliser cette fonction car le systeme n'est pas pret");
				return false;
			}

			$cache = $this->addr_vers_tmp_file($addr);

			if ($update != 0) # On transforme une premiere fois l'update s'il est specifie
				$update = $this->transformer_update($update);

			if ($update > 0) { # On regarde si l'update arrive a echeance
				if (@file_exists($cache) && @is_file($cache) && @is_readable($cache)) {
					# Un fichier cache existe
					if ( (time() - filemtime($cache)) <= $update )
						$addr = $cache; # On lit le cache
				}
			}
			$update = 0;

			if ($update == 0) { # Lecture du fichier xml distant ou local
				$data = @file_get_contents($addr);
				if (!$data) { // false ou equivalent
					if ($this->trigger_errors) trigger_error("EasyRSS::lire_xml Erreur : impossible de recuperer le fichier distant");

					if (@file_exists($cache) && @is_file($cache) && @is_readable($cache)) {
						# Si un cache existe, on l'utilise puisque la connexion a echouee
						$addr = $cache;
						$data = @file_get_contents($cache);
						if (!$data) {
							# Si a marche pas on supprime le cache et on quitte
							@unlink($data);
							return false;
						}
					}
					else return false;
				}
				# On parse le fichier xml
				# C'est artisanal mais bon a marche tres bien
				$r = array();
				$tmp = preg_split("/<\/?".$item.">/", $data);
				for ($i=1; $i < sizeof($tmp)-1; $i+=2) {
					 foreach($champs as $champ) {
						$tmp2 = preg_split("/<\/?".$champ.">/", $tmp[$i]);
						if ($result_type == "ASSOC") $r[$i-1][$champ] = @$tmp2[1];
						else $r[$i-1][] = @$tmp2[1];
					 }
				}

				# On enregistre tout a dans un fichier cache, si on vient de faire une connexion distante
				if ($addr != $cache) {

					@unlink($cache);
					$fp = @fopen($cache, 'w+');
					if ($fp) {
						if (@!fwrite($fp, $data)) {
							if ($trigger_errors) trigger_error("EasyRSS::lire_xml Erreur : impossible d'ecrire dans le fichier de cache");
						}
						@fclose($fp);
					}
					else if ($trigger_errors) trigger_error("EasyRSS::lire_xml Erreur : impossible d'ouvrir le fichier de cache");

				}
				# Sortie
				return $r;
			}
		}

		function addr_vers_tmp_file($addr, $limit=128) {
			# Caracteres interdits supprimes
			$a = array("\\", ':', '*', '?', '"', '<', '>', '|', '!', ';', ',', "\t", "\r", "\n", "\0", "\x0B");
			$b = array();
			foreach ($a as $c) $b[] = "";
			$addr = str_replace($a, $b, $addr);
			# Caracteres remplaces
			$a = array('/', '.', ' ', '+');
			$b = array('-', '_', '_', '-');
			$addr = str_replace($a, $b, $addr);
			# Caracteres speciaux remplaces
			$a = "";
			$b = "aaaaaaaceeeeiiiidnoooooouuuuybsaaaaaaaceeeeiiiidnoooooouuuyybycr_"; 
			$addr = strtr(trim($addr), $a, $b);
			# Sortie
			$addr .= '.tmp';
			if (strlen($addr) > $limit)
				$addr = substr($addr, (strlen($addr) - $limit), $limit);
			return $this->temp_dir.'/'.$addr;
		}

		function transformer_update($temps) {
			list($v, $u) = explode(" ", $temps);
			$v = intval($v);
			switch ($u) {
				case "sec" : case "secs" :
					return $v; break;
				case "min" : case "mins" :
					return $v * 60; break;
				case "hour" : case "hours" :
					return $v * 60 * 60; break;
				case "day" : case "days" :
					return $v * 60 * 60 * 24; break;
				case "week" : case "weeks" :
					return $v * 60 * 60 * 24 * 7; break;
				case "month" : case "months" :
					return $v * 60 * 60 * 24 * 7 * 30; break;
				case "year" : case "years" :
					return $v * 60 * 60 * 24 * 365; break;
				default : return 0;
			}			
		}

		# Fin de la classe
	}
?>